# -*- coding: utf-8 -*-

from copy import deepcopy
from tools import trace, timer

from PyQt5.QtWidgets import (
    QMessageBox, QUndoCommand, QUndoStack,
)

from Model.Section.Section import Section
from Model.Section.SectionList import SectionList

class SetNameCommand(QUndoCommand):
    def __init__(self, sections, index, new_value):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).name
        self._new = new_value

    def undo(self):
        self._sections.get(self._index).name = self._old

    def redo(self):
        self._sections.get(self._index).name = self._new

class SetBeginCommand(QUndoCommand):
    def __init__(self, sections, index, new_value):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).begin_kp
        self._new = new_value

    def undo(self):
        self._sections.get(self._index).begin_kp = float(self._old)

    def redo(self):
        self._sections.get(self._index).begin_kp = float(self._new)

class SetEndCommand(QUndoCommand):
    def __init__(self, sections, index, new_value):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).end_kp
        self._new = new_value

    def undo(self):
        self._sections.get(self._index).end_kp = float(self._old)

    def redo(self):
        self._sections.get(self._index).end_kp = float(self._new)


class SetBeginStricklerCommand(QUndoCommand):
    def __init__(self, sections, index, new_value):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).begin_strickler
        self._new = new_value

        print(self._old)
        print(self._new)

    def undo(self):
        self._sections.get(self._index).begin_strickler = self._old

    def redo(self):
        self._sections.get(self._index).begin_strickler = self._new

class SetEndStricklerCommand(QUndoCommand):
    def __init__(self, sections, index, new_value):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).end_strickler
        self._new = new_value

        print(self._old)
        print(self._new)

    def undo(self):
        self._sections.get(self._index).end_strickler = self._old

    def redo(self):
        self._sections.get(self._index).end_strickler = self._new


class SetEdgeCommand(QUndoCommand):
    def __init__(self, sections, index, edge):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._old = self._sections.get(self._index).edge
        self._new = edge

    def undo(self):
        self._sections.get(self._index).edge = self._old

    def redo(self):
        self._sections.get(self._index).edge = self._new

class AddCommand(QUndoCommand):
    def __init__(self, sections, index):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._index = index
        self._new = None

    def undo(self):
        self._sections.delete_i([self._index])

    def redo(self):
        if self._new is None:
            self._new = self._sections.new(self._index)
        else:
            self._sections.insert(self._index, self._new)

class DelCommand(QUndoCommand):
    def __init__(self, sections, rows):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._rows = rows

        self._section = []
        for row in rows:
            self._section.append((row, self._sections.get(row)))
        self._section.sort()

    def undo(self):
        for row, el in self._section:
            self._sections.insert(row, el)

    def redo(self):
        self._sections.delete_i(self._rows)

class SortCommand(QUndoCommand):
    def __init__(self, sections, _reverse):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._reverse = _reverse

        self._old = self._sections.sections
        self._indexes = None

    def undo(self):
        ll = self._sections.sections
        self._sections.sort(
            key=lambda x: self._indexes[ll.index(x)]
        )

    def redo(self):
        self._sections.sort(
            reverse=self._reverse,
            key=lambda x: x.name
        )
        if self._indexes is None:
            self._indexes = list(
                map(
                    lambda p: self._old.index(p),
                    self._sections.sections
                )
            )
            self._old = None


class MoveCommand(QUndoCommand):
    def __init__(self, sections, up, i):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._up = up == "up"
        self._i = i

    def undo(self):
        if self._up:
            self._sections.move_up(self._i)
        else:
            self._sections.move_down(self._i)

    def redo(self):
        if self._up:
            self._sections.move_up(self._i)
        else:
            self._sections.move_down(self._i)


class PasteCommand(QUndoCommand):
    def __init__(self, sections, row, section):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._row = row
        self._section = deepcopy(section)
        self._section.reverse()

    def undo(self):
        self._sections.delete(self._section)

    def redo(self):
        for section in self._section:
            self._sections.insert(self._row, section)


class DuplicateCommand(QUndoCommand):
    def __init__(self, sections, rows, section):
        QUndoCommand.__init__(self)

        self._sections = sections
        self._rows = rows
        self._section = deepcopy(section)
        self._section.reverse()

    def undo(self):
        self._sections.delete(self._section)

    def redo(self):
        for section in self._sections:
            self._sections.insert(self._rows[0], section)
