# -*- coding: utf-8 -*-

from tools import trace, timer

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
)

from View.Sections.UndoCommand import (
    PasteCommand, DuplicateCommand,
)

from View.Sections.Table import (
    TableModel, ComboBoxDelegate
)

from View.Plot.MplCanvas import MplCanvas
from View.Geometry.PlotKPC import PlotKPC
from View.Sections.translate import *

_translate = QCoreApplication.translate

class SectionsWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Sections", study=None, parent=None):
        title = title + " - " + study.name

        super(SectionsWindow, self).__init__(
            name=title, ui="Sections", parent=parent
        )

        self._study = study
        self._sections = self._study.river.sections

        self.setup_sc()
        self.setup_table()
        self.setup_graph()
        self.setup_connections()

        self.ui.setWindowTitle(title)

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        self._table = {}

        table = self.find(QTableView, f"tableView")
        self._table = TableModel(
            data = self._study.river,
            undo = self._undo_stack,
        )
        table.setModel(self._table)

        self._delegate_edge = ComboBoxDelegate(
            data = self._study.river,
            mode = "edge",
            parent=self
        )
        self._delegate_stricklers = ComboBoxDelegate(
            data = self._study.river,
            mode = "stricklers",
            parent=self
        )

        table.setItemDelegateForColumn(
            1, self._delegate_edge
        )
        table.setItemDelegateForColumn(
            4, self._delegate_stricklers
        )
        table.setItemDelegateForColumn(
            5, self._delegate_stricklers
        )

        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_graph(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.plot_layout = self.find(QVBoxLayout, "verticalLayout")
        self.plot_layout.addWidget(self.canvas)

        self.plot = PlotKPC(
            canvas = self.canvas,
            data = None,
            toolbar = None,
            display_current = False
        )

    def setup_connections(self):
        self.find(QAction, "action_add").triggered.connect(self.add)
        self.find(QAction, "action_del").triggered.connect(self.delete)
        self.find(QAction, "action_sort").triggered.connect(self.sort)

        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

        table = self.find(QTableView, f"tableView")
        table.selectionModel()\
             .selectionChanged\
             .connect(self._set_current_reach)

        self._table.dataChanged\
                   .connect(self._set_current_reach)

    def index_selected_rows(self):
        table = self.find(QTableView, f"tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )

    def _set_current_reach(self):
        rows = self.index_selected_rows()

        data = None
        highlight = None

        if len(rows) > 0:
            edge = self._study\
                       .river\
                       .sections\
                       .get(rows[0])\
                       .edge
            if edge:
                data = edge.reach
                sec = self._sections.get(rows[0])
                highlight = (sec.begin_kp, sec.end_kp)

        self.plot = PlotKPC(
            canvas = self.canvas,
            data = data,
            toolbar = None,
            display_current = False
        )
        self.plot.draw(highlight=highlight)

    def add(self):
        rows = self.index_selected_rows()
        if len(self._sections) == 0 or len(rows) == 0:
            self._table.add(0)
        else:
            self._table.add(rows[0])

    def delete(self):
        rows = self.index_selected_rows()
        if len(rows) == 0:
            return

        self._table.delete(rows)

    def sort(self):
        self._table.sort(False)

    def move_up(self):
        row = self.index_selected_row()
        self._table.move_up(row)

    def move_down(self):
        row = self.index_selected_row()
        self._table.move_down(row)

    def copy(self):
        print("TODO")

    def paste(self):
        print("TODO")

    def undo(self):
        self._table.undo()

    def redo(self):
        self._table.redo()
